<?php


namespace MetatickPluginLib\License;


class License
{
    protected $host = null;
    protected $name = null;
    protected $secret = null;
    protected $slug = null;
    protected $network = false;

    protected $config = [];

    protected $default = [
        'error' => false,
        'license' => '',
        'expire' => '0000-00-00',
        'verify' => '',
        'features' => []
    ];

    public function __construct($host, $name, $secret, $slug, $network = false){
        $this->host = $host;
        $this->name = $name;
        $this->secret = $secret;
        $this->slug = $slug;
        $this->network = $network;

        $this->load();

        add_action('admin_notices', [$this,'admin_notices']);
        add_action('admin_init', [$this, 'admin_init']);
        add_action("{$slug}_license_check", [$this, 'check']);

        $this->configure_schedule();
    }

    public function configure_schedule(){
        if(!wp_next_scheduled( "{$this->slug}_license_check")){
            wp_schedule_event(time(), 'daily', "{$this->slug}_license_check");
        }
    }

    public function admin_init(){
        add_action("wp_ajax_{$this->slug}_license_activate", [$this, 'wp_ajax_activate']);
        add_action("wp_ajax_{$this->slug}_license_deactivate", [$this, 'wp_ajax_deactivate']);
    }

    public function admin_notices(){
        if($this->is_licensed()){
            return;
        }
        $message = "{$this->name} is unlicensed";

        if($this->has_error()) {
            $message .= ': ' . $this->get_error();
        }

        ?>
        <div class="notice notice-error is-dismissible">
            <p><?php echo $message; ?></p>
        </div>
        <?php
    }

    public function load(){
        if($this->network){
            $config = get_site_option("{$this->slug}_license", []);
        }else {
            $config = get_option("{$this->slug}_license", []);
        }

        $this->config = array_merge($this->default, $config);
    }

    public function save(){
        if($this->network) {
            update_site_option("{$this->slug}_license", $this->config);
        }else {
            update_option("{$this->slug}_license", $this->config, true);
        }
    }

    public function set_license($license){
        $this->activate($license);
    }

    function get_license(){
        return $this->config['license'];
    }

    public function activate($license){
        $this->config['license'] = $license;
        $this->save();
        $this->check();
    }

    function get_expire(){
        return $this->config['expire'];
    }

    function get_verify(){
        return $this->config['verify'];
    }

    function get_error(){
        $license = $this->get_license();
        if(!$license || $license == ''){
            return "No License Specified";
        }
        return $this->config['error'];
    }

    function has_error(){
        $error = $this->get_error();

        if($error != null)
            return ! empty($error);

        return false;
    }

    function get_features(){
        return $this->config['features'];
    }

    function get_uid(){
        if($this->network){
            $data = AUTH_KEY;
        }else{
            $data = AUTH_KEY . get_current_blog_id();
        }
        return strtolower(md5(strtolower($data)));
    }

    function get_device(){
        return get_site_url();
    }

    function is_licensed(){
        $data = $this->get_uid() . $this->get_license() . $this->secret . $this->get_expire();
        $hash = strtolower(md5(strtolower($data)));
        return $hash === $this->get_verify();
    }

    function check(){
        try{
            $url = $this->host . "/api/v1/wordpress/activate/";
            $response = wp_remote_post(
                $url,
                array(
                    'method' => 'POST',
                    'timeout' => 45,
                    'redirection' => 5,
                    'blocking' => true,
                    'sslverify' => false,
                    'body' => array(
                        'slug' => $this->slug,
                        'key' => $this->get_license(),
                        'uid' => $this->get_uid(),
                        'device' => $this->get_device()
                    ),
                )
            );

            if(is_wp_error($response) && !$this->is_licensed()){
                $this->config['error'] = $response->get_error_message();
                $this->save();
                return;
            }

            $message = json_decode(wp_remote_retrieve_body($response));
            if(isset($message->result)){
                if($message->result === 'success'){
                    $options = array(
                        'verify',
                        'features',
                        'expire',
                        'updates',
                        'support',
                        'error'
                    );

                    foreach($options as $option){
                        if(property_exists($message, $option)){
                            $this->config[$option] = $message->$option;
                        }
                    }
                    $this->save();
                }
            }else {
                throw new \Exception("Invalid Response");
            }
        }catch (\Exception $e){
            if(!$this->is_licensed()){
                $this->config['error'] = $e->getMessage();
                $this->save();
            }
        }
    }

    function has_feature($check){
        try {
            $features = $this->get_features();
            if(is_object($features)){
                if(property_exists($features, $check)){
                    return true;
                }
            }
            if(is_array($features)){
                if(array_key_exists($check, $features)){
                    return true;
                }
                if(in_array($check, $features)){
                    return true;
                }
            }
        }catch (\Exception $e){
            return false;
        }
        return false;
    }

    function wp_ajax_activate(){
        if ( !current_user_can('manage_options') ) {
            echo "Not allowed";
        }
        if(!array_key_exists('key', $_POST)){
            echo 'No Key Received!';
        }else{
            $options = array(
                'verify',
                'features',
                'expire',
                'updates',
                'support',
                'error'
            );

            $this->config['license'] = stripslashes($_POST['key']);

            foreach($options as $option){
                if(array_key_exists($option, $_POST)){
                    $this->config[$option] = stripslashes($_POST[$option]);
                }
            }
            $this->save();
            $this->check();

            if($this->is_licensed()){
                echo 'Activated';
            }elseif ($this->has_error()){
                echo $this->get_error();
            }else{
                echo 'An Unknown Error Occurred';
            }
        }
        exit;
    }

    function wp_ajax_deactivate(){
        if ( !current_user_can('manage_options') ) {
            echo "Not allowed";
        }
        $this->config['license'] = $this->config['verify'] = '';
        $this->save();
        echo 'License Deactivated';
        exit;
    }

    function get_obfuscated_key(){
        $key = $this->get_license();
        $length = strlen($key);
        $start_length = 0;
        $end_length = 0;
        if($length >= 4 && $length < 8){
            $start_length = 1;
            $end_length = 1;
        }elseif ($length >= 8){
            $start_length = 2;
            $end_length = 2;
        }
        $start = substr($key, 0, $start_length);
        $end = substr($key, 0-$end_length);

        return $start . str_repeat('*', $length - strlen($start) - strlen($end)) . $end;

    }

    function get_status() {
        if ($this->is_licensed()) {
            $status = 'Activated (' . $this->get_obfuscated_key() . ')';
        }else{
            $status = 'Not Licensed';
        }
        return apply_filters("{$this->slug}_license_status", $status, $this);
    }

    function has_support(){
        if(!array_key_exists('support', $this->config)){
            $has_support = false;
        }else {
            $support = $this->config['support'];
            $has_support = $support >= time();
        }
        return apply_filters("{$this->slug}_license_has_support", $has_support, $this);
    }

    function has_updates(){
        if(!array_key_exists('updates', $this->config)){
            $has_updates = false;
        } else {
            $updates = $this->config['updates'];
            $has_updates = $updates >= time();
        }
        return apply_filters("{$this->slug}_license_has_updates", $has_updates, $this);
    }

    function get_includes(){
        $includes = array();
        if($this->has_support()){
            $includes[] = 'Support';
        }

        if($this->has_updates()){
            $includes[] = 'Updates';
        }

        if(count($includes) == 0){
            $includes = '';
        }else {
            $includes = 'Includes ' . implode(' and ', $includes);
        }
        return apply_filters("{$this->slug}_license_includes", $includes, $this);
    }

    function get_actions(){
        if ($this->is_licensed()) {
            $actions = "<button class='button' id='{$this->slug}_license_deactivate' data-product='$this->slug'>Deactivate License</button>";
        }else{
            $actions = "<button class='button' id='{$this->slug}_license_activate' data-product='$this->slug'>Activate License</button>";
        }

        return apply_filters("{$this->slug}_license_actions", $actions, $this);
    }

    function get_description(){
        return apply_filters("{$this->slug}_license_description", 'Software License', $this);
    }

    function get_javascript_activation_url(){
        $parameters = array(
            'slug' => $this->slug,
            'uid' => $this->get_uid(),
            'device' => $this->get_device()
        );

        return $this->host . "/api/v1/wordpress/jsactivate/?" . http_build_query($parameters);
    }

    function widget($echo=false){
        $status = $this->get_status();
        $actions = $this->get_actions();

        if($this->is_licensed()) {
            $description = $this->get_description();
            $includes = $this->get_includes();

            $rows = array(
                'status' => $status,
                'description' => $description,
                'includes' => $includes,
                'actions' => $actions,
            );
        }else{
            $rows = array(
                'status' => $status,
                'actions' => $actions,
            );
        }
        ob_start();
        ?>
        <div class="license-widget license-widget-<?php echo $this->slug; ?>">
            <?php foreach($rows as $name => $content): ?>
                <?php if($content and strlen($content) > 0): ?>
                    <div class="license-<?php echo $name; ?>"><?php echo $content; ?></div>
                <?php endif; ?>
            <?php endforeach; ?>
        </div>
        <style>
            .license-widget {
                position: relative;
                padding: 20px;
                background-color: white;
                border: 1px solid #d2d2d2;
                margin: 1px;
                max-width: 25em;
                box-sizing: border-box;
            }
            .license-status,
            .license-description,
            .license-includes {
                margin-bottom:5px;
            }
            .license-description {
                font-weight: bold;
            }
            .license-includes {
                opacity: 0.8;
            }
            .license-actions {
                text-align: right;
            }
        </style>
        <script>
            (function($){
                var ajax_url = <?php echo json_encode(admin_url( 'admin-ajax.php' )); ?>;

                $('#<?php echo $this->slug; ?>_license_activate').click(function(e){
                    <?php if(array_key_exists('allow_key_entry', $_GET)): ?>
                    var key = prompt('Enter your <?php echo $this->name; ?> key:');
                    if(key){
                        $.post(
                            ajax_url, {
                                'key': key,
                                'action': '<?php echo $this->slug; ?>_license_activate'
                            },
                            function(response) {
                                location.reload();
                            }
                        );
                    }
                    <?php else: ?>
                    var url = <?php echo json_encode($this->get_javascript_activation_url()); ?>;
                    var specs = 'height=400,width=400,left=50,location=no,menubar=no,top=50';
                    var licensing_window = window.open(url, 'license', specs);
                    window.addEventListener(
                        "message",
                        function(message){
                            try {
                                if(message.data
                                    && message.data.action
                                    && message.data.action === 'activate'
                                    && message.data.product
                                    && message.data.product === '<?php echo $this->slug; ?>'){

                                    $.post(
                                        ajax_url, {
                                            'key': message.data.key,
                                            'verify': message.data.verify,
                                            'features': message.data.features,
                                            'expire': message.data.expire,
                                            'updates': message.data.updates,
                                            'support': message.data.support,
                                            'error': message.data.error,
                                            'action': '<?php echo $this->slug; ?>_license_activate'
                                        },
                                        function(response) {
                                            licensing_window.close();
                                            location.reload();
                                        }
                                    );
                                }
                            }catch(e){
                                console.log('Unhandled error: ' + e);
                            }
                        },
                        false
                    );

                    <?php endif; ?>
                    e.preventDefault();
                });
                $('#<?php echo $this->slug; ?>_license_deactivate').click(function(e){
                    if(confirm("Are you sure you wish to deactivate <?php echo $this->name; ?>?")) {
                        $.post(
                            ajax_url, {
                                'action': '<?php echo $this->slug; ?>_license_deactivate'
                            },
                            function(response) {
                                alert(response);
                                location.reload();
                            }
                        );
                    }
                    e.preventDefault();
                });
            })(jQuery);
        </script>
        <?php
        $result = ob_get_clean();
        if($echo){
            echo $result;
        }
        return $result;
    }
}